@@ -6,6 +6,7 @@ import functools
 import subprocess
 import time
 import traceback
+
 from abc import ABC, abstractmethod
 from typing import Any, Callable, Dict, List, Optional
 
@@ -54,9 +55,7 @@ def hoppr_process(func: Callable) -> Callable:
         if comp is not None:
             arg_string = f"(purl: {comp.purl})"
 
-        self.get_logger().debug(
-            f"Starting {self.__class__.__name__}.{func.__name__} {arg_string}"
-        )
+        self.get_logger().debug(f"Starting {self.__class__.__name__}.{func.__name__} {arg_string}")
 
         result = None
         if comp is not None:
@@ -67,13 +66,9 @@ def hoppr_process(func: Callable) -> Callable:
                 purl_type = PackageURL.from_string(comp.purl).type
 
                 if not self.supports_purl_type(purl_type):
-                    return Result.skip(
-                        f"Class {self.__class__.__name__} does not support purl type {purl_type}"
-                    )
+                    return Result.skip(f"Class {self.__class__.__name__} does not support purl type {purl_type}")
 
-                component_header = f"{comp.name}" + (
-                    f"@{comp.version}" if comp.version is not None else ""
-                )
+                component_header = f"{comp.name}" + (f"@{comp.version}" if comp.version is not None else "")
 
                 self.get_logger().info(
                     msg=f"{'-' * 4} Component: {component_header} {'-' * 50}",
@@ -83,9 +78,7 @@ def hoppr_process(func: Callable) -> Callable:
 
             # Only check for missing commands if func has been overridden
             if func.__module__ != HopprPlugin.__module__:
-                command_result = plugin_utils.check_for_missing_commands(
-                    self.required_commands
-                )
+                command_result = plugin_utils.check_for_missing_commands(self.required_commands)
                 if command_result.is_fail():
                     self.get_logger().error(command_result.message)
                     return command_result
@@ -126,19 +119,15 @@ def hoppr_rerunner(method: Callable) -> Callable:
 
         log = self.get_logger()
 
+        result = Result.fail(message="Context max_attempts must be a positive integer.")
+
         for attempt in range(self.context.max_attempts):
-            log.info(
-                msg=f"Processing component [attempt {attempt + 1} of {self.context.max_attempts}]",
-                indent_level=1,
-            )
+            log.info(msg=f"Processing component [attempt {attempt + 1} of {self.context.max_attempts}]", indent_level=1)
 
             result = method(self, *args, **kwargs)
 
             if not isinstance(result, Result):
-                msg = (
-                    f"Method {method.__name__} returned {type(result).__name__} "
-                    + "in rerunner. Result object required"
-                )
+                msg = f"Method {method.__name__} returned {type(result).__name__} in rerunner. Result object required"
 
                 log.error(msg=msg, indent_level=1)
                 return Result.fail(message=msg)
@@ -148,31 +137,18 @@ def hoppr_rerunner(method: Callable) -> Callable:
 
             if attempt < self.context.max_attempts - 1:
                 log.warning(
-                    msg=(
-                        f"Method {method.__name__} will be retried in "
-                        + f"{self.context.retry_wait_seconds} seconds"
-                    ),
+                    msg=(f"Method {method.__name__} will be retried in {self.context.retry_wait_seconds} seconds"),
                     indent_level=1,
                 )
 
-                log.warning(
-                    msg=f"Result message for attempt {attempt + 1}: {result.message}",
-                    indent_level=1,
-                )
+                log.warning(msg=f"Result message for attempt {attempt + 1}: {result.message}", indent_level=1)
 
                 time.sleep(self.context.retry_wait_seconds)
 
-        log.error(
-            msg=f"Method {method.__name__} failed after {self.context.max_attempts} attempts",
-        )
+        log.error(msg=f"Method {method.__name__} failed after {self.context.max_attempts} attempts")
+        log.error(msg=f"Result message for final attempt: {result.message}")
 
-        log.error(
-            msg=f"Result message for final attempt: {result.message}",
-        )
-
-        return Result.fail(
-            f"Failure after {self.context.max_attempts} attempts, final message {result.message}"
-        )
+        return Result.fail(f"Failure after {self.context.max_attempts} attempts, final message {result.message}")
 
     return wrapper
 
@@ -186,6 +162,19 @@ class HopprPlugin(ABC):
 
     required_commands: List[str] = []
     supported_purl_types: List[str] = []
+
+    """
+    Product: the result of carrying out a step. Products are usually persistent (e.g,.
+    files), and are often meant to be used as materials on subsequent steps. Products are
+    recorded as part of link metadata.
+
+    This is a list of strings where each element is a relative path file or wildcard.
+
+    See in-toto specificaion:
+    - https://github.com/in-toto/docs/blob/master/in-toto-spec.md
+    """
+    products: List[str] = []
+
     default_component_coverage = ComponentCoverage.OPTIONAL
     bom_access = BomAccess.NO_ACCESS
 
@@ -231,23 +220,22 @@ class HopprPlugin(ABC):
         """
         return Result.skip("post_stage_process not defined.")
 
-    @classmethod
-    def supports_purl_type(cls, purl_type: str) -> bool:
+    def supports_purl_type(self, purl_type: str) -> bool:
         """
         Indicates whether or not this particular plug-in supports components of the specified
         purl type.
 
-        If a plug-in supports all purl types, this method can be overridden to always return True.
+        If no supported purl types are defined, the plug-in supports all purl types.
         """
-        return purl_type in cls.supported_purl_types
+        return purl_type in self.supported_purl_types or not self.supported_purl_types
 
     def create_logger(self) -> None:
         """
         Creates a logger with a MemoryHandler, which flushes to stderr when closed
         """
         self._logger = MemoryLogger(
-            self.context.logfile_location,
-            self.context.logfile_lock,
+            file_name=self.context.logfile_location,
+            lock=self.context.logfile_lock,
             log_level=self.context.log_level,
         )
 
@@ -263,38 +251,24 @@ class HopprPlugin(ABC):
         """
         self._logger.close()
 
-    def run_command(
-        self, command, password_list=None, cwd=None
-    ) -> subprocess.CompletedProcess[bytes]:
+    def run_command(self, command, password_list=None, cwd=None) -> subprocess.CompletedProcess[bytes]:
         """
         Run a command and log any errors
         """
         obscured_command = obscure_passwords(command, password_list)
-        self.get_logger().debug(
-            msg=f"Running command: '{obscured_command}'", indent_level=2
-        )
+        self.get_logger().debug(msg=f"Running command: '{obscured_command}'", indent_level=2)
 
         result = subprocess.run(command, check=False, capture_output=True, cwd=cwd)
 
         if result.returncode != 0:
             # Prefix each line of the command's stdout with 4 spaces
-            stdout_indented = "\n".join(
-                ["    " + line for line in result.stdout.decode("utf-8").split("\n")]
-            )
+            stdout_indented = "\n".join(["    " + line for line in result.stdout.decode("utf-8").split("\n")])
 
-            self.get_logger().debug(
-                msg=f"{command[0]} command stdout content:\n{stdout_indented}",
-                indent_level=2,
-            )
+            self.get_logger().debug(msg=f"{command[0]} command stdout content:\n{stdout_indented}", indent_level=2)
 
             # Prefix each line of the command's stderr with 4 spaces
-            stderr_indented = "\n".join(
-                ["    " + line for line in result.stderr.decode("utf-8").split("\n")]
-            )
+            stderr_indented = "\n".join(["    " + line for line in result.stderr.decode("utf-8").split("\n")])
 
-            self.get_logger().error(
-                msg=f"{command[0]} command failed with error:\n{stderr_indented}",
-                indent_level=2,
-            )
+            self.get_logger().error(msg=f"{command[0]} command failed with error:\n{stderr_indented}", indent_level=2)
 
         return result
