@@ -49,13 +49,14 @@ class CollectDockerPlugin(SerialCollectorPlugin):
 
         source_image = os.path.join(repo_url, purl.namespace or "", purl.name + ":" + purl.version)
         source_image = re.sub(r"^https?://", "", source_image)
-        if not source_image.startswith("docker://"):
-            source_image = "docker://" + source_image
 
         file_name = urllib.parse.unquote(purl.name) + "_" + purl.version
         target_dir = self.directory_for(purl.type, repo_url, subdir=purl.namespace)
         target_path = os.path.join(target_dir, file_name)
-        destination = "docker-archive:" + target_path
+        destination = f'docker-archive:{target_path}:{source_image}'
+
+        if not source_image.startswith("docker://"):
+            source_image = "docker://" + source_image
 
         self.get_logger().info(
             msg="Copying docker image:",
