@@ -5,7 +5,8 @@ Collector plugin for git repositories
 import os
 import pathlib
 import re
-from typing import Any, Dict, Optional
+
+from typing import Any, Dict, List, Optional
 
 from packageurl import PackageURL  # type: ignore
 
@@ -24,6 +25,7 @@ class CollectGitPlugin(SerialCollectorPlugin):
 
     supported_purl_types = ["git", "gitlab", "github"]
     required_commands = ["git"]
+    products: List[str] = ["git/*", "gitlab/*", "github/*"]
 
     def get_version(self) -> str:  # pylint: disable=duplicate-code
         return __version__
@@ -51,9 +53,7 @@ class CollectGitPlugin(SerialCollectorPlugin):
             indent_level=2,
         )
 
-        target_dir = self.directory_for(
-            purl.type, repo_url, subdir=os.path.join(purl.namespace, purl.name)
-        )
+        target_dir = self.directory_for(purl.type, repo_url, subdir=os.path.join(purl.namespace, purl.name))
 
         git_result = self.git_clone(target_dir, source_url, creds)
         if not git_result.is_success():
