@@ -41,6 +41,32 @@ class CollectPypiPlugin(SerialCollectorPlugin):
         self.password_list: list[str] = []
         self.base_command = [self.required_commands[0], "-m", "pip"]
 
+    def _run_cmd_wrapper(  # pylint: disable=too-many-arguments
+        self, command, password_list, param: str, pkg_format: str, log_if_error: str
+    ) -> bool:
+        """
+        Run command utility for discrete function calls - required for DO-178C Level-A branch isolation coverage
+        """
+        run_result = self.run_command([*command, param], password_list)
+        try:
+            run_result.check_returncode()
+            return True
+        except CalledProcessError:
+            self.get_logger().debug(msg=f"{log_if_error} {pkg_format} package", indent_level=2)
+        return False
+
+    def collect_binary_only(self, command, password_list, log_if_error: str) -> bool:
+        """
+        Only collect the PyPI binary (WHL)
+        """
+        return self._run_cmd_wrapper(command, password_list, "--only-binary=:all:", "binary", log_if_error)
+
+    def collect_source(self, command, password_list, log_if_error: str) -> bool:
+        """
+        Only collect the PyPI source package
+        """
+        return self._run_cmd_wrapper(command, password_list, "--no-binary=:all:", "source", log_if_error)
+
     @hoppr_rerunner
     def collect(self, comp: Component, repo_url: str, creds: CredObject | None = None) -> Result:
         """
@@ -84,24 +110,11 @@ class CollectPypiPlugin(SerialCollectorPlugin):
             f"{purl.name}=={purl.version}",
         ]
 
-        got_binary = False
         base_error_msg = f"Failed to download {purl.name} version {purl.version}"
 
-        run_result = self.run_command([*command, "--only-binary=:all:"], password_list)
-
-        try:
-            run_result.check_returncode()
-            got_binary = True
-        except CalledProcessError:
-            self.get_logger().debug(msg=f"{base_error_msg} binary package", indent_level=2)
-
-        run_result = self.run_command([*command, "--no-binary=:all:"], password_list)
-
-        try:
-            run_result.check_returncode()
-        except CalledProcessError:
-            self.get_logger().debug(msg=f"{base_error_msg} source package", indent_level=2)
-            if not got_binary:
-                return Result.retry(f"{base_error_msg} as either whl or source.")
-
-        return Result.success()
+        if self.collect_binary_only(command, password_list, base_error_msg):  # pylint: disable=no-else-return
+            return Result.success()
+        elif self.collect_source(command, password_list, base_error_msg):  # pylint: disable=no-else-return
+            return Result.success()
+        else:
+            return Result.retry(f"{base_error_msg} as either binary or source.")
