@@ -2,12 +2,11 @@
 Enumeration to indicate how a plugin may change a BOM
 """
 import enum
+
 from typing import Any
 
-from hoppr_cyclonedx_models.cyclonedx_1_4 import (
-    Component,
-    CyclonedxSoftwareBillOfMaterialsStandard as Bom,
-)  # type: ignore
+from hoppr_cyclonedx_models.cyclonedx_1_4 import Component
+from hoppr_cyclonedx_models.cyclonedx_1_4 import CyclonedxSoftwareBillOfMaterialsStandard as Bom  # type: ignore
 
 
 class BomAccess(enum.IntEnum):
