@@ -10,17 +10,18 @@ class PurlType(Enum):
     Enumeration of supported purl types
     """
 
+    DEB = "deb"
     DOCKER = "docker"
+    GENERIC = "generic"
     GIT = "git"
     GITHUB = "github"
     GITLAB = "gitlab"
     GOLANG = "golang"
     HELM = "helm"
-    RPM = "rpm"
-    PYPI = "pypi"
     MAVEN = "maven"
-    GENERIC = "generic"
-    APT = "apt"
+    NPM = "npm"
+    PYPI = "pypi"
+    RPM = "rpm"
 
     def __str__(self) -> str:
         return self.value
