@@ -2,16 +2,17 @@
 Run Hoppr processing, using multiple processors
 """
 from __future__ import annotations
-from copy import deepcopy
 
 import logging
+import multiprocessing
 import os
-import platform
 import shutil
 import socket
 import tempfile
 import time
+
 from concurrent.futures import Future, ProcessPoolExecutor
+from copy import deepcopy
 from datetime import datetime
 from itertools import zip_longest
 from multiprocessing import cpu_count
@@ -20,10 +21,8 @@ from pathlib import Path
 from typing import Any, Dict, List, Optional, Tuple, Union
 from urllib.parse import quote_plus
 
-from hoppr_cyclonedx_models.cyclonedx_1_4 import (
-    Component,
-    CyclonedxSoftwareBillOfMaterialsStandard as Bom,
-)  # type: ignore
+from hoppr_cyclonedx_models.cyclonedx_1_4 import Component
+from hoppr_cyclonedx_models.cyclonedx_1_4 import CyclonedxSoftwareBillOfMaterialsStandard as Bom  # type: ignore
 from packageurl import PackageURL  # type: ignore
 from typer import colors, echo, secho
 
@@ -37,18 +36,18 @@ from hoppr.flatten_sboms import flatten_sboms
 from hoppr.hoppr_types.bom_access import BomAccess
 from hoppr.hoppr_types.component_coverage import ComponentCoverage
 from hoppr.hoppr_types.transfer_file_content import Stage as StageRef
+from hoppr.in_toto import HopprInTotoLinks
 from hoppr.mem_logger import MemoryLogger
 from hoppr.net import download_file
 from hoppr.result import Result
 from hoppr.utils import plugin_instance
 
-if "nux" in platform.system():
-    import pwd  # pylint: disable=import-error
+# Import on Unix systems only
+if os.name == "posix":
+    import pwd
 
 
-def _run_plugin(
-    plugin: HopprPlugin, method_name: str, component: Optional[Component]
-) -> Result:
+def _run_plugin(plugin: HopprPlugin, method_name: str, component: Optional[Component]) -> Result:
     """
     Runs a single method for a single component (if supplied) on a single plugin
     """
@@ -84,9 +83,7 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
         self.plugin_ref_list = stage_ref.plugins
         self.config_component_coverage = None
         if stage_ref.component_coverage is not None:
-            self.config_component_coverage = ComponentCoverage[
-                stage_ref.component_coverage
-            ]
+            self.config_component_coverage = ComponentCoverage[stage_ref.component_coverage]
         self.required_coverage = ComponentCoverage.OPTIONAL
         self.plugins: List[HopprPlugin] = []
         self.results: Dict = {}
@@ -134,18 +131,14 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
                     # Create one concurrent future object to run this method for each component
 
                     for component in self.context.delivered_sbom.components or []:
-                        future_proc = executor.submit(
-                            _run_plugin, plugin, method_name, component
-                        )
+                        future_proc = executor.submit(_run_plugin, plugin, method_name, component)
                         future_argument_map[future_proc] = (plugin, component)
                         futures.append(future_proc)
                 else:
 
                     # Create a concurrent future object to run this method
 
-                    future_proc = executor.submit(
-                        _run_plugin, plugin, method_name, None
-                    )
+                    future_proc = executor.submit(_run_plugin, plugin, method_name, None)
                     future_argument_map[future_proc] = (plugin, None)
                     futures.append(future_proc)
 
@@ -162,9 +155,7 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
                         echo(f"   Beginning method {method_name}")
                         need_method_label = False
                     plugin, comp = future_argument_map[future_proc]
-                    self._save_result(
-                        method_name, plugin.__class__.__name__, future_result, comp
-                    )
+                    self._save_result(method_name, plugin.__class__.__name__, future_result, comp)
                     self._report_result(plugin.__class__.__name__, comp, future_result)
                     self._update_bom(future_result.return_obj, comp)
 
@@ -178,18 +169,14 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
 
         return self._get_stage_result(method_name, failures, retries)
 
-    def _get_stage_result(
-        self, method_name: str, failures: int, retries: int
-    ) -> Result:
+    def _get_stage_result(self, method_name: str, failures: int, retries: int) -> Result:
         if failures + retries == 0:
             return Result.success()
         if failures == 0 and retries > 0:
             return Result.fail(f"{retries} '{method_name}' processes returned 'retry'")
         if failures > 0 and retries == 0:
             return Result.fail(f"{failures} '{method_name}' processes failed")
-        return Result.fail(
-            f"{failures} '{method_name}' processes failed, and {retries} returned 'retry'"
-        )
+        return Result.fail(f"{failures} '{method_name}' processes failed, and {retries} returned 'retry'")
 
     def _get_required_coverage(self) -> ComponentCoverage:
         if self.config_component_coverage is not None:
@@ -221,12 +208,8 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
                 bad_comp_result = Result.fail(
                     f"Component processed {count} times, {self.required_coverage.name} coverage required"
                 )
-                self._save_result(
-                    method_name, f"Stage {self.stage_id}", bad_comp_result, component
-                )
-                self._report_result(
-                    f"Stage {self.stage_id}", component, bad_comp_result
-                )
+                self._save_result(method_name, f"Stage {self.stage_id}", bad_comp_result, component)
+                self._report_result(f"Stage {self.stage_id}", component, bad_comp_result)
                 additional_failures += 1
 
         return additional_failures
@@ -249,10 +232,7 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
             secho(msg, fg=colors.RED)
             return Result.fail(msg)
 
-        if (
-            len(access_counts[BomAccess.COMPONENT_ACCESS]) > 0
-            and self.required_coverage.max_value > 1
-        ):
+        if len(access_counts[BomAccess.COMPONENT_ACCESS]) > 0 and self.required_coverage.max_value > 1:
             msg = (
                 f"Stage {self.stage_id} has one or more plugins with {BomAccess.COMPONENT_ACCESS.name}: "
                 + ", ".join(access_counts[BomAccess.COMPONENT_ACCESS])
@@ -266,12 +246,8 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
         return Result.success()
 
     def _update_bom(self, return_obj: Any, comp: Component | None):
-        if self.context.delivered_sbom.components is not None and isinstance(
-            return_obj, Component
-        ):
-            for index, delivered_comp in enumerate(
-                self.context.delivered_sbom.components
-            ):
+        if self.context.delivered_sbom.components is not None and isinstance(return_obj, Component):
+            for index, delivered_comp in enumerate(self.context.delivered_sbom.components):
                 if delivered_comp == comp:
                     self.context.delivered_sbom.components[index] = return_obj
                     break
@@ -287,6 +263,7 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
         color = colors.GREEN
         if not result.is_success():
             color = colors.RED
+        if result.message:
             desc = desc + f": {result.message}"
         secho(desc, fg=color)
 
@@ -325,36 +302,42 @@ class StageProcessor:  # pylint: disable=too-few-public-methods
 
         plugin_list = []
         for plugin_ref in self.plugin_ref_list:
-            plugin_inst = plugin_instance(
-                plugin_ref.name, self.context, plugin_ref.config
-            )
+            plugin_inst = plugin_instance(plugin_ref.name, self.context, plugin_ref.config)
             if (
                 plugin_inst.supported_purl_types == []
-                or len(used_purl_types.intersection(plugin_inst.supported_purl_types))
-                > 0
+                or len(used_purl_types.intersection(plugin_inst.supported_purl_types)) > 0
             ):
                 plugin_list.append(plugin_inst)
 
         return plugin_list
 
 
-class HopprProcessor:  # pylint: disable=too-few-public-methods
+class HopprProcessor:  # pylint: disable=too-few-public-methods,too-many-instance-attributes
     """
     Run the Hoppr process
     """
 
-    def __init__(
-        self, transfer: Transfer, manifest: Manifest, log_level: int = logging.INFO
+    def __init__(  # pylint: disable=too-many-arguments
+        self,
+        transfer: Transfer,
+        manifest: Manifest,
+        create_attestations: bool = False,
+        functionary_key_path: Optional[Path] = None,
+        functionary_key_password: Optional[str] = None,
+        log_level: int = logging.INFO,
     ) -> None:
         self.transfer = transfer
         self.manifest = manifest
         self.context: Context
+        self.logger: MemoryLogger
+        self.log_level = log_level
         self.stages: Dict = {}
         self.metadata_files: List[Path] = []
-        self.logger: Optional[MemoryLogger] = None
-        self.log_level = log_level
+        self.in_toto_links = HopprInTotoLinks(
+            create_attestations, self.transfer, functionary_key_path, functionary_key_password
+        )
 
-    def get_logger(self):
+    def get_logger(self) -> MemoryLogger:
         """
         Returns the logger for this class
         """
@@ -371,33 +354,21 @@ class HopprProcessor:  # pylint: disable=too-few-public-methods
         target = Path(target_dir, quote_plus(f"{abs_path}"))
         shutil.copyfile(file_name, f"{target}")
 
-    def _collect_url(
-        self, url: Union[str, PathLike[str]], target_dir: Union[str, PathLike[str]]
-    ) -> None:
+    def _collect_url(self, url: Union[str, PathLike[str]], target_dir: Union[str, PathLike[str]]) -> None:
         self.get_logger().info(f"Collecting metadata from url {url}")
 
         target = Path(target_dir, quote_plus(f"{url}"))
         download_file(url, f"{target}", None)
 
-    def _collect_oci_artifact(
-        self, artifact: str, target_dir: Union[str, PathLike[str]]
-    ):
+    def _collect_oci_artifact(self, artifact: str, target_dir: Union[str, PathLike[str]]) -> Path:
         target = os.path.join(target_dir, quote_plus(artifact))
         self.get_logger().info(f"Collecting metadata from OCI {artifact} to {target}")
-        return oci_artifacts.pull_artifact_to_disk(
-            artifact, target, allow_version_discovery=True
-        )
+        return oci_artifacts.pull_artifact_to_disk(artifact, target, allow_version_discovery=True)
 
-    def _collect_manifest_metadata(
-        self, manifest: Manifest, target_dir: Union[str, PathLike[str]]
-    ) -> None:
-        for (child_ref, child) in zip_longest(
-            manifest.manifest_file_content.includes, manifest.children
-        ):
+    def _collect_manifest_metadata(self, manifest: Manifest, target_dir: Union[str, PathLike[str]]) -> None:
+        for (child_ref, child) in zip_longest(manifest.manifest_file_content.includes, manifest.children):
             if child_ref.local is not None:
-                child_path = Path(manifest.manifest_location).parent.joinpath(
-                    child_ref.local
-                )
+                child_path = Path(manifest.manifest_location).parent.joinpath(child_ref.local)
                 self._collect_file(child_path, target_dir)
             else:
                 self._collect_url(f"{child_ref.url}", Path(target_dir))
@@ -406,9 +377,7 @@ class HopprProcessor:  # pylint: disable=too-few-public-methods
 
         for bom_ref in manifest.manifest_file_content.sbom_refs:
             if bom_ref.local is not None:
-                sbom_file_path = Path(manifest.manifest_location).parent.joinpath(
-                    bom_ref.local
-                )
+                sbom_file_path = Path(manifest.manifest_location).parent.joinpath(bom_ref.local)
                 self._collect_file(sbom_file_path, target_dir)
             elif bom_ref.oci is not None:
                 self._collect_oci_artifact(bom_ref.oci, target_dir)
@@ -420,8 +389,8 @@ class HopprProcessor:  # pylint: disable=too-few-public-methods
         target_dir = Path(self.context.collect_root_dir, "generic", "_metadata_")
         target_dir.mkdir(exist_ok=True, parents=True)
 
-        if "nux" in platform.system():
-            user = pwd.getpwuid(os.getuid())[0]  # pylint: disable=no-member
+        if os.name == "posix":
+            user = pwd.getpwuid(os.getuid()).pw_name
         else:
             user = os.getlogin()
 
@@ -439,19 +408,24 @@ class HopprProcessor:  # pylint: disable=too-few-public-methods
 
         self._collect_manifest_metadata(self.context.manifest, target_dir)
 
-    def _collect_bom(self) -> None:
+    def _collect_consolidated_bom(self) -> None:
         target_dir = Path(self.context.collect_root_dir, "generic", "_metadata_")
         target_dir.mkdir(parents=True, exist_ok=True)
 
-        with open(
-            target_dir.joinpath("_consolidated_bom.json"), "w", encoding="utf-8"
-        ) as bom_data:
-            bom_data.write(self.context.consolidated_sbom.json(by_alias=True, indent=2))
+        with open(target_dir.joinpath("_consolidated_bom.json"), "w", encoding="utf-8") as bom_data:
+            bom_data.write(self.context.consolidated_sbom.json(exclude_none=True, by_alias=True, indent=2))
 
-        with open(
-            target_dir.joinpath("_delivered_bom.json"), "w", encoding="utf-8"
-        ) as bom_data:
-            bom_data.write(self.context.delivered_sbom.json(by_alias=True, indent=2))
+    def _collect_delivered_bom(self, stage_name: Optional[str] = None) -> None:
+        target_dir = Path(self.context.collect_root_dir, "generic", "_metadata_")
+        target_dir.mkdir(parents=True, exist_ok=True)
+
+        if stage_name:
+            preface = f"intermediate_{stage_name}_"
+        else:
+            preface = ""
+
+        with open(target_dir.joinpath(f"_{preface}delivered_bom.json"), "w", encoding="utf-8") as bom_data:
+            bom_data.write(self.context.delivered_sbom.json(exclude_none=True, by_alias=True, indent=2))
 
     def _summarize_results(self) -> int:
         """
@@ -482,18 +456,14 @@ class HopprProcessor:  # pylint: disable=too-few-public-methods
 
                 total_failure_count += failure_count
                 total_success_count += result_count - failure_count
-                echo(
-                    f"      {result_count - failure_count} jobs succeeded, {failure_count} failed"
-                )
+                echo(f"      {result_count - failure_count} jobs succeeded, {failure_count} failed")
                 if failure_count > 0:
                     echo(failure_list)
 
-        echo(
-            f"\nGRAND TOTAL: {total_success_count} jobs succeeded, {total_failure_count} failed\n"
-        )
+        echo(f"\nGRAND TOTAL: {total_success_count} jobs succeeded, {total_failure_count} failed\n")
         return total_failure_count
 
-    def run(self, log_file: Optional[Path] = None) -> Result:
+    def run(self, log_file: Optional[Path] = None, strict_repos: bool = True) -> Result:
         """
         Run the Hoppr process executing each stage in turn
         """
@@ -503,7 +473,7 @@ class HopprProcessor:  # pylint: disable=too-few-public-methods
 
             flattened_bom = flatten_sboms(self.manifest)
             if len(flattened_bom.components or []) == 0:
-                msg = "No SBOMs defined in manifests, or SBOMs contain no components.  Nothing to process."
+                msg = "No SBOMs defined in manifests, or SBOMs contain no components. Nothing to process."
                 secho(msg, fg=colors.RED)
                 return Result.fail(msg)
 
@@ -514,55 +484,61 @@ class HopprProcessor:  # pylint: disable=too-few-public-methods
                 delivered_sbom=deepcopy(flattened_bom),
                 max_processes=cpu_count(),
                 log_level=self.log_level,
+                strict_repos=strict_repos,
+                logfile_lock=multiprocessing.Manager().RLock(),
             )
+
             if self.transfer.content:
                 self.context.max_processes = self.transfer.content.max_processes
 
-            logger_name: str = f"HopprProcessor--{os.getpid()}"
+            self.context.logfile_location = f"hoppr_{time.strftime('%Y%m%d-%H%M%S')}.log"
 
-            self.context.logfile_location = (
-                f"hoppr_{time.strftime('%Y%m%d-%H%M%S')}.log"
-            )
             if log_file is not None:
                 self.context.logfile_location = str(log_file)
 
             self.logger = MemoryLogger(
                 self.context.logfile_location,
                 lock=self.context.logfile_lock,
-                log_name=logger_name,
+                log_name=f"HopprProcessor--{os.getpid()}",
                 log_level=self.context.log_level,
                 flush_immed=True,
             )
 
-            self.get_logger().info(
-                f"Beginning Hoppr Process execution, max_processes={self.context.max_processes}"
-            )
-            echo(
-                f"Beginning Hoppr Process execution, max_processes={self.context.max_processes}"
-            )
+            msg = f"Beginning Hoppr Process execution, max_processes={self.context.max_processes}"
+            self.get_logger().info(msg=msg)
+            echo(message=msg)
+
+            self.in_toto_links.set_collection_root(collection_root)
+            self.in_toto_links.record_stage_start("_collect_metadata")
             self._collect_metadata()
+            self._collect_consolidated_bom()
+            self.in_toto_links.record_stage_stop("_collect_metadata")
 
             if self.transfer.content is not None:
                 for stage_ref in self.transfer.content.stages:
-                    self.get_logger().info(
-                        f"{'=' * 10} Beginning Stage {stage_ref.name} {'=' * 50}"
-                    )
+                    self.get_logger().info(f"{'=' * 10} Beginning Stage {stage_ref.name} {'=' * 50}")
                     echo(f"{'=' * 10} Beginning Stage {stage_ref.name} {'=' * 50}")
 
                     stage = StageProcessor(stage_ref, self.context)
                     self.stages[stage_ref.name] = stage
+
+                    self.in_toto_links.record_stage_start(stage_ref.name)
                     result = stage.run()
+
                     if result.is_fail() or result.is_retry():
-                        self.logger.error(
-                            f"Stage {stage_ref.name} failed, processing terminated: {result.message}"
-                        )
-                        secho(
-                            f"   Stage {stage_ref.name} failed, processing terminated: {result.message}",
-                            fg=colors.RED,
-                        )
+                        msg = f"Stage {stage_ref.name} failed, processing terminated: {result.message}"
+                        self.get_logger().error(msg=msg)
+                        secho(message=f"  {msg}", fg=colors.RED)
+
                         break
 
-                    self._collect_bom()
+                    self._collect_delivered_bom(stage_ref.name)
+                    self._collect_delivered_bom()
+                    self.in_toto_links.record_stage_stop(stage_ref.name)
+
+                self.in_toto_links.record_stage_start("_finalize")
+                self._collect_delivered_bom()
+                self.in_toto_links.record_stage_stop("_finalize")
 
         failed_jobs = self._summarize_results()
         if failed_jobs > 0:
