@@ -3,6 +3,7 @@ Class to store results of processes
 """
 
 import enum
+
 from typing import Any
 
 import requests
@@ -118,9 +119,7 @@ class Result:
 
         match response.status_code:
             case response_code if 200 <= response_code <= 299:  # pylint: disable=used-before-assignment
-                return Result.success(
-                    f"HTTP Status Code: {response.status_code}", return_obj=return_obj
-                )
+                return Result.success(f"HTTP Status Code: {response.status_code}", return_obj=return_obj)
             case response_code if 500 <= response_code:
                 status = ResultStatus.RETRY
             case _:
